@@ -27,6 +27,7 @@
 #include "tqdbusdatalist.h"
 #include "tqdbusdatamap.h"
 #include "tqdbusobjectpath.h"
+#include "tqdbusunixfd.h"
 #include "tqdbusvariant.h"
 
 #include <tqshared.h>
@@ -50,6 +51,10 @@ public:
                 delete (TQT_DBusObjectPath*)value.pointer;
                 break;
 
+            case TQT_DBusData::UnixFd:
+                delete (TQT_DBusUnixFd*)value.pointer;
+                break;
+
             case TQT_DBusData::List:
                 delete (TQT_DBusDataList*)value.pointer;
                 break;
@@ -101,8 +106,12 @@ public:
                         delete (TQT_DBusDataMap<TQT_DBusObjectPath>*)value.pointer;
                         break;
 
+                    case TQT_DBusData::UnixFd:
+                        delete (TQT_DBusDataMap<TQT_DBusUnixFd>*)value.pointer;
+                        break;
+
                     default:
-                        qFatal("TQT_DBusData::Private: unhandled map key type %d(%s)",
+                        tqFatal("TQT_DBusData::Private: unhandled map key type %d(%s)",
                                keyType, TQT_DBusData::typeName(keyType));
                         break;
                 }
@@ -132,6 +141,38 @@ public:
     } value;
 };
 
+// key type definitions for TQT_DBusDataMap
+template <>
+const TQT_DBusData::Type TQT_DBusDataMap<TQ_UINT8>::m_keyType = TQT_DBusData::Byte;
+
+template <>
+const TQT_DBusData::Type TQT_DBusDataMap<TQ_INT16>::m_keyType = TQT_DBusData::Int16;
+
+template <>
+const TQT_DBusData::Type TQT_DBusDataMap<TQ_UINT16>::m_keyType = TQT_DBusData::UInt16;
+
+template <>
+const TQT_DBusData::Type TQT_DBusDataMap<TQ_INT32>::m_keyType = TQT_DBusData::Int32;
+
+template <>
+const TQT_DBusData::Type TQT_DBusDataMap<TQ_UINT32>::m_keyType = TQT_DBusData::UInt32;
+
+template <>
+const TQT_DBusData::Type TQT_DBusDataMap<TQ_INT64>::m_keyType = TQT_DBusData::Int64;
+
+template <>
+const TQT_DBusData::Type TQT_DBusDataMap<TQ_UINT64>::m_keyType = TQT_DBusData::UInt64;
+
+template <>
+const TQT_DBusData::Type TQT_DBusDataMap<TQString>::m_keyType = TQT_DBusData::String;
+
+template <>
+const TQT_DBusData::Type TQT_DBusDataMap<TQT_DBusObjectPath>::m_keyType = TQT_DBusData::ObjectPath;
+
+template <>
+const TQT_DBusData::Type TQT_DBusDataMap<TQT_DBusUnixFd>::m_keyType = TQT_DBusData::UnixFd;
+
+
 TQT_DBusData::TQT_DBusData() : d(new Private())
 {
 }
@@ -208,6 +249,9 @@ bool TQT_DBusData::operator==(const TQT_DBusData& other) const
             case TQT_DBusData::ObjectPath:
                 return toObjectPath() == other.toObjectPath();
 
+            case TQT_DBusData::UnixFd:
+                return toUnixFd() == other.toUnixFd();
+
             case TQT_DBusData::List:
                 return toList() == other.toList();
 
@@ -223,42 +267,37 @@ bool TQT_DBusData::operator==(const TQT_DBusData& other) const
                 switch (d->keyType)
                 {
                     case TQT_DBusData::Byte:
-                        toByteKeyMap() == other.toByteKeyMap();
-                        break;
+                        return toByteKeyMap() == other.toByteKeyMap();
 
                     case TQT_DBusData::Int16:
-                        toInt16KeyMap() == other.toInt16KeyMap();
-                        break;
+                        return toInt16KeyMap() == other.toInt16KeyMap();
 
                     case TQT_DBusData::UInt16:
-                        toUInt16KeyMap() == other.toUInt16KeyMap();
-                        break;
+                        return toUInt16KeyMap() == other.toUInt16KeyMap();
 
                     case TQT_DBusData::Int32:
-                        toInt32KeyMap() == other.toInt32KeyMap();
-                        break;
+                        return toInt32KeyMap() == other.toInt32KeyMap();
 
                     case TQT_DBusData::UInt32:
-                        toUInt32KeyMap() == other.toUInt32KeyMap();
-                        break;
+                        return toUInt32KeyMap() == other.toUInt32KeyMap();
+
                     case TQT_DBusData::Int64:
-                        toInt64KeyMap() == other.toInt64KeyMap();
-                        break;
+                        return toInt64KeyMap() == other.toInt64KeyMap();
 
                     case TQT_DBusData::UInt64:
-                        toUInt64KeyMap() == other.toUInt64KeyMap();
-                        break;
+                        return toUInt64KeyMap() == other.toUInt64KeyMap();
 
                     case TQT_DBusData::String:
-                        toStringKeyMap() == other.toStringKeyMap();
-                        break;
+                        return toStringKeyMap() == other.toStringKeyMap();
 
                     case TQT_DBusData::ObjectPath:
-                        toObjectPathKeyMap() == other.toObjectPathKeyMap();
-                        break;
+                        return toObjectPathKeyMap() == other.toObjectPathKeyMap();
+
+                    case TQT_DBusData::UnixFd:
+                        return toUnixFdKeyMap() == other.toUnixFdKeyMap();
 
                     default:
-                        qFatal("TQT_DBusData operator== unhandled map key type %d(%s)",
+                        tqFatal("TQT_DBusData operator== unhandled map key type %d(%s)",
                                d->keyType, TQT_DBusData::typeName(d->keyType));
                         break;
                 }
@@ -303,6 +342,7 @@ const char* TQT_DBusData::typeName(Type type)
         case TQT_DBusData::Double:     return "Double";
         case TQT_DBusData::String:     return "String";
         case TQT_DBusData::ObjectPath: return "ObjectPath";
+        case TQT_DBusData::UnixFd:     return "UnixFd";
         case TQT_DBusData::List:       return "List";
         case TQT_DBusData::Struct:     return "Struct";
         case TQT_DBusData::Variant:    return "Variant";
@@ -568,6 +608,32 @@ TQT_DBusObjectPath TQT_DBusData::toObjectPath(bool* ok) const
     return *((TQT_DBusObjectPath*)d->value.pointer);
 }
 
+TQT_DBusData TQT_DBusData::fromUnixFd(const TQT_DBusUnixFd& value)
+{
+    TQT_DBusData data;
+
+    if (value.isValid())
+    {
+        data.d->type = TQT_DBusData::UnixFd;
+        data.d->value.pointer = new TQT_DBusUnixFd(value);
+    }
+
+    return data;
+}
+
+TQT_DBusUnixFd TQT_DBusData::toUnixFd(bool* ok) const
+{
+    if (d->type != TQT_DBusData::UnixFd)
+    {
+        if (ok != 0) *ok = false;
+        return TQT_DBusUnixFd();
+    }
+
+    if (ok != 0) *ok = true;
+
+    return *((TQT_DBusUnixFd*)d->value.pointer);
+}
+
 TQT_DBusData TQT_DBusData::fromList(const TQT_DBusDataList& list)
 {
     TQT_DBusData data;
@@ -885,6 +951,31 @@ TQT_DBusDataMap<TQT_DBusObjectPath> TQT_DBusData::toObjectPathKeyMap(bool* ok) c
     return *((TQT_DBusDataMap<TQT_DBusObjectPath>*)d->value.pointer);
 }
 
+TQT_DBusData TQT_DBusData::fromUnixFdKeyMap(const TQT_DBusDataMap<TQT_DBusUnixFd>& map)
+{
+    TQT_DBusData data;
+
+    data.d->type = TQT_DBusData::Map;
+    data.d->keyType = map.keyType();
+    data.d->value.pointer = new TQT_DBusDataMap<TQT_DBusUnixFd>(map);
+
+    return data;
+}
+
+TQT_DBusDataMap<TQT_DBusUnixFd> TQT_DBusData::toUnixFdKeyMap(bool* ok) const
+{
+    if (d->type != TQT_DBusData::Map &&
+        d->keyType != TQT_DBusDataMap<TQT_DBusUnixFd>::m_keyType)
+    {
+        if (ok != 0) *ok = false;
+        return TQT_DBusDataMap<TQT_DBusUnixFd>();
+    }
+
+    if (ok != 0) *ok = true;
+
+    return *((TQT_DBusDataMap<TQT_DBusUnixFd>*)d->value.pointer);
+}
+
 static const char* qDBusTypeForTQT_DBusType(TQT_DBusData::Type type)
 {
     switch (type)
@@ -913,6 +1004,8 @@ static const char* qDBusTypeForTQT_DBusType(TQT_DBusData::Type type)
             return DBUS_TYPE_STRING_AS_STRING;
         case TQT_DBusData::ObjectPath:
             return DBUS_TYPE_OBJECT_PATH_AS_STRING;
+        case TQT_DBusData::UnixFd:
+            return DBUS_TYPE_UNIX_FD_AS_STRING;
         case TQT_DBusData::Variant:
             return DBUS_TYPE_VARIANT_AS_STRING;
         default:
@@ -1008,6 +1101,10 @@ TQCString TQT_DBusData::buildDBusSignature() const
                     signature += qDBusSignatureForMapValue<TQT_DBusObjectPath>(
                         *((TQT_DBusDataMap<TQT_DBusObjectPath>*) d->value.pointer));
                     break;
+                case TQT_DBusData::UnixFd:
+                    signature += qDBusSignatureForMapValue<TQT_DBusUnixFd>(
+                        *((TQT_DBusDataMap<TQT_DBusUnixFd>*) d->value.pointer));
+                    break;
                 default:
                     break;
             }
@@ -1022,33 +1119,3 @@ TQCString TQT_DBusData::buildDBusSignature() const
 
     return signature;
 }
-
-
-// key type definitions for TQT_DBusDataMap
-template <>
-const TQT_DBusData::Type TQT_DBusDataMap<TQ_UINT8>::m_keyType = TQT_DBusData::Byte;
-
-template <>
-const TQT_DBusData::Type TQT_DBusDataMap<TQ_INT16>::m_keyType = TQT_DBusData::Int16;
-
-template <>
-const TQT_DBusData::Type TQT_DBusDataMap<TQ_UINT16>::m_keyType = TQT_DBusData::UInt16;
-
-template <>
-const TQT_DBusData::Type TQT_DBusDataMap<TQ_INT32>::m_keyType = TQT_DBusData::Int32;
-
-template <>
-const TQT_DBusData::Type TQT_DBusDataMap<TQ_UINT32>::m_keyType = TQT_DBusData::UInt32;
-
-template <>
-const TQT_DBusData::Type TQT_DBusDataMap<TQ_INT64>::m_keyType = TQT_DBusData::Int64;
-
-template <>
-const TQT_DBusData::Type TQT_DBusDataMap<TQ_UINT64>::m_keyType = TQT_DBusData::UInt64;
-
-template <>
-const TQT_DBusData::Type TQT_DBusDataMap<TQString>::m_keyType = TQT_DBusData::String;
-
-template <>
-const TQT_DBusData::Type TQT_DBusDataMap<TQT_DBusObjectPath>::m_keyType =
-        TQT_DBusData::ObjectPath;
